﻿using HeuristicLab.Clients.Access;
using HeuristicLab.Clients.Common;
using HeuristicLab.Clients.Common.Properties;
using HeuristicLab.Clients.Hive.WebJobManager.Services;
using Microsoft.AspNet.Mvc;
using System.ServiceModel.Security;

namespace HeuristicLab.Clients.Hive.WebJobManager.Controllers
{
    /// <summary>
    /// Controller for initial landing page
    /// </summary>
    public class HomeController : Controller
    {
        private LoginViewModelService loginViewModelService;
        private HiveServiceClient client;
        public HomeController(ILoginViewModelService loginViewModelService)
        {
            this.loginViewModelService = (LoginViewModelService)loginViewModelService;
        }
        #region Login
        /// <summary>
        /// Opens initial home page
        /// </summary>
        /// <returns>View from home page</returns>
        public IActionResult Index()
        {
            ViewBag.Title = "Login";
            loginViewModelService.clear();
            return View(loginViewModelService.GetLoginViewModel());
        }
        /// <summary>
        /// Checks login
        /// </summary>
        /// <param name="loginName">Login name</param>
        /// <param name="password">Login password</param>
        /// <returns>Logged in view if correct or shows error</returns>
        public IActionResult Login(string loginName, string password)//Checks login
        {
            if (!string.IsNullOrEmpty(loginName) && !string.IsNullOrEmpty(password))
            {
                var model = loginViewModelService.GetLoginViewModel();
                HiveServiceLocatorWeb hiveServiceLocator = (HiveServiceLocatorWeb)HiveServiceLocatorWeb.Instance;
                HeuristicLab.Clients.Common.Properties.Settings.Default.UserName = loginName;
                HeuristicLab.Clients.Common.Properties.Settings.Default.Password = Common.CryptoService.EncryptString(password);
                HeuristicLab.Clients.Common.Properties.Settings.Default.Save();
                hiveServiceLocator.Username = loginName;
                hiveServiceLocator.Password = password;

                client = hiveServiceLocator.getHiveServiceClient();
                try {
                    var test = client.GetJobs();//Throws messageSecurityException if login failss
                    ViewBag.Title = "Login succesful";
                    
                    return RedirectToAction("Index","Job");
                }
                catch(MessageSecurityException e)
                {
                    ViewBag.Title = "Login";
                    model.errorMessage = "Wrong login, try again";
                    return View("Index", model);
                }

            }
            else
            {
                ViewBag.Title = "Login";
                var model = loginViewModelService.GetLoginViewModel();
                model.errorMessage = "You should fill in both fields";
                return View("Index", model);
            }
        }
        public IActionResult Logout()
        {
            return RedirectToAction("Index","Home");
        }
        #endregion
    }
}
